-- sim_kb_input.vhd
--
-- testbench for module kb_input
-- lab "Digitaltechnik "
--
-- (c) M. Winzker, Hochschule Bonn-Rhein-Sieg, 06.05.2016

library ieee;
use ieee.std_logic_1164.all;

entity sim_kb_input is
end sim_kb_input;

architecture sim of sim_kb_input is

signal clk      : std_logic := '0';
signal reset    : std_logic;
signal kbclk    : std_logic;
signal kbdata   : std_logic;
signal valid    : std_logic;
signal byte     : std_logic_vector(7 downto 0);

begin

-- clock generation
clk <= not clk after 20 ns;

-- instantiation of design-under-verification
duv: entity work.kb_input
    port map (  clk     => clk,
                reset   => reset,
                kbclk   => kbclk,
                kbdata  => kbdata,
                byte    => byte,
                valid   => valid);

process
    variable dataword : std_logic_vector( 7 downto 0);
    variable parity   : std_logic;
    variable ps2_word : std_logic_vector(10 downto 0);
    variable i, k     : integer;
begin

-- init with reset='1' for 2 clock cycles
reset   <= '1', '0' after 80 ns;
-- clock and data are '1' if inactive
kbclk   <= '1';
kbdata  <= '1';

-- start simulation, wait 50 clock cycles
for i in 0 to 49 loop
    wait until falling_edge(clk);
end loop;

dataword := to_stdlogicvector(bit_vector'(X"1C")); -- dataword to be transmitted

-- with the dataword, the complete PS/2 word with 11 bits is constructed
parity := '1';  -- calculate odd parity, i.e. the number of 1's in the data bits plus
                -- the parity bit always add up to an odd number.
for k in 0 to 7 loop
    parity  := parity xor dataword(k);
end loop;
ps2_word( 0)          := '0';        -- startbit '0'
ps2_word( 8 downto 1) := dataword;   -- content of tranmission
ps2_word( 9)          := parity;     -- parity
ps2_word(10)          := '1';        -- stopbit '1'

-- loop to transmit the complete dataword (least significant bit first)
for k in 0 to 10 loop -- for all 11 bits
    kbdata  <= ps2_word(k);                     -- set kbdata
    for i in 0 to 4 loop
        wait until falling_edge(clk); end loop; -- wait  5 clock cycles
    kbclk   <= '0';                             -- set kbclk to '0'
    for i in 0 to 9 loop
        wait until falling_edge(clk); end loop; -- wait 10 clock cycles
    kbclk   <= '1';                             -- set kbclk to '1'
    for i in 0 to 4 loop
        wait until falling_edge(clk); end loop; -- wait 5 clock cycles
end loop; -- loop k, for all 11 bits

-- wait 100 clock cycles until end of simulation
for i in 0 to 99 loop
    wait until falling_edge(clk);
end loop;

-- stop simulation
assert false
    report "Simulation completed"
    severity failure;

end process;

end sim;

